/* This file is part of Cloudy and is copyright (C)1978-2007 by Gary J. Ferland
 * For conditions of distribution and use see copyright notice in license.txt */
/* */
/* unset EXTERN so that everything is defined here */
#define EXTERN 
#include "cddefines.h"

/* used for punching map*/
FILE *ioMAP = NULL;

/* external ZeroNum used to div by zero 
 * ok here since never changed*/
const double ZeroNum = 0.;

#include "taulines.h"/* this must go here since it defines NTA needed for other lines*/

/* following is true extern in taulines.h */
long nWindLine = NWINDDIM;

/* set number of CO rotation levels to default, 20 as set in taulines.h */
long int nCORotate = NCOROTATE;

/* say that space not allocated yet */
bool lgCORotateMalloc = false;

#include "abund.h"
#include "atmdat.h"
#include "atoms.h"
#include "atomfeii.h"
#include "assertresults.h"
#include "broke.h"
#include "ca.h"
#include "called.h"
#include "carb.h"
#include "cddrive.h"
/* this will be set true when cdInit is called.  The definition is in cdInit.
 * Other routines will check that this is true when they are called, 
 * to verify that cdInit was called first */
bool lgcdInitCalled=false;
#include "colden.h"
#include "conv.h"
#include "continuum.h"
#include "coolheavy.h"
#include "dense.h"
#include "doppvel.h"
#include "dynamics.h"
#include "elementnames.h"
#include "embesq.h"
#include "extinc.h"
#include "fe.h"
#include "fudgec.h"
#include "geometry.h"
#include "grainvar.h"
/* optimize must come here because header needed in grid.h */
#include "optimize.h"
/*char chOptimFileName[] = "optimal.in";*/
#include "grid.h"
#include "h2.h"
#include "h2_priv.h"
bool lgH2_READ_DATA = false;
#include "helike.h" 
#include "he.h"
#include "heavy.h"
#include "hextra.h"
#include "hmi.h"
#include "hydrogenic.h"

/* this is set true once space malloced, then never change
 * number of levels again with hydrogenic command, 
 * it is used in gethydro to never set number of levels one set here,
 * also to make sure MALLOC only happens one time  */
bool lgHydroMalloc = false;

#include "hyperfine.h"
#include "input.h"
#include "ionbal.h"
#include "iso.h"
#include "iterations.h"
#include "lines.h"
/* these are the definitions of the line save arrays in lines.h */
/*LinDstSv *LineDSv=NULL;*/
LinSv *LineSv=NULL;
#include "magnetic.h"
#include "map.h"
#include "mean.h"
#include "mewecoef.h"
#include "mole.h" 
#include "neutrn.h"
#include "nitro.h"
#include "noexec.h"
#include "numderiv.h"
#include "oxy.h"
#include "peimbt.h"
#include "phycon.h"
#include "plot.h"
#include "sil.h"
#include "version.h"
/* this is set true when space is allocated for the FeII arrays,
 * once this happens FeII.nFeIILevel cannot be changed with the atom feii levels command */
bool lgFeIIMalloc=false;
/* */
#include "pressure.h"
#include "prt.h"
#include "punch.h"
#include "radius.h"
#include "reason.h"
#include "rfield.h"
/* set true when malloced, init to false */
bool lgRfieldMalloced=false;
#include "opacity.h"
bool lgOpacMalloced=false;
#include "rt.h"
#include "secondaries.h"
#include "state.h"
#include "stopcalc.h"
#include "struc.h"
#include "thermal.h"
#include "timesc.h"
#include "trace.h"
#include "warnings.h"
#include "wind.h"

void cdDefines(void)
{
	long int nelem;

	strncpy( chOptimFileName , "optimal.in" , sizeof( chOptimFileName ) );

	/* only reset number of levels if space has not been allocated - in all single run
	 * models this will be false, so levels will be set - exception is grid of models,
	 * this will only happen in first call */
	if( !lgHydroMalloc )
	{

		/* top off for he-like sequence */
		/** \todo	2	do something with this or set to LONG_MAX? 
		* after moving collapsed levels to iso.nCollapsed_max, this 
		* variable does nothing.	
		iso.nTopOff[ipHE_LIKE] = 0;*/

		/* variables for H-like sequence */
		/* default number of levels for hydrogen atom */
		for( nelem=ipHYDROGEN; nelem < LIMELM; ++nelem )
		{
			iso.n_HighestResolved_max[ipH_LIKE][nelem] = 15;
			/* >>chng 02 jan 18, increase by one to include 2s, so that all loops will be to < nLevels */
			iso.numLevels_max[ipH_LIKE][nelem] = iso.n_HighestResolved_max[ipH_LIKE][nelem] + 1;
			/* >>chng 03 jun 06, put number of top off to one less than number,
			 * since intend to have more levels */
			iso.nTopOff[ipH_LIKE][nelem] = iso.numLevels_max[ipH_LIKE][nelem]-1;

			/* number of levels to print */
			iso.numPrintLevels[ipH_LIKE][nelem] = iso.numLevels_max[ipH_LIKE][nelem];
			/* number of collapsed levels */
			iso.nCollapsed_max[ipH_LIKE][nelem] = 0;
		}
		for( nelem=ipHYDROGEN; nelem <= ipHELIUM; ++nelem )
		{
			/* >>chng 03 jun 01 up number of levels for H and He from 15 to 26 */
			iso.n_HighestResolved_max[ipH_LIKE][nelem] = 25;
			/* >>chng 02 jan 18, increase by one to include 2s, so that all loops will be to < nLevels */
			iso.numLevels_max[ipH_LIKE][nelem] = iso.n_HighestResolved_max[ipH_LIKE][nelem] + 1;

			/* lowest level is 2s,
			* in versions 90 this was equal to 10, but was 1 in 91
			* >>chng 99 jan 16, changed back to 10*/
			/*iso.nTopOff[ipH_LIKE] = 1;*/
			/* >>chng 03 jun 06, put number of topoff to one less than number,
			 * since intend to have more levels */
			/*iso.nTopOff[ipH_LIKE][nelem] = iso.numLevels_max[ipH_LIKE][nelem]-1;*/
			/* >>chng 03 oct 16, from above to 22 to avoid maser near topoff */
			iso.nTopOff[ipH_LIKE][nelem] = 22;

			/* number of levels to print */
			iso.numPrintLevels[ipH_LIKE][nelem] = iso.numLevels_max[ipH_LIKE][nelem] - 1;
		}

		/* total number of He-like levels, hydrogen does not exist, of course */
		iso.n_HighestResolved_max[ipHE_LIKE][ipHYDROGEN] = 0;
		iso.numLevels_max[ipHE_LIKE][ipHYDROGEN] = 0;
		iso.numPrintLevels[ipHE_LIKE][ipHYDROGEN] = 0;
		iso.nCollapsed_max[ipHE_LIKE][ipHYDROGEN] = 0;

		/* these are the total number of l levels, including the resolved 2 3P */
		for( nelem=ipHELIUM; nelem < LIMELM; ++nelem )
		{
			/* put at least three resolved and 1 collapsed in every element for he-like */
			iso.n_HighestResolved_max[ipHE_LIKE][nelem] = 3;
			/** \todo	2	Change this to higher number. 
			 * Make this change by itself after all other changes pass suite.  */
			iso.nCollapsed_max[ipHE_LIKE][nelem] = 1;
		}

		/* >>chng 05 dec 21, from 10 to 20, greater precision without too much more time */
		iso.nCollapsed_max[ipHE_LIKE][ipHELIUM] = 20;
		/* >>chng 05 dec 21, make 6 rather than n=7 default max resolved for Helium itself */
		iso.n_HighestResolved_max[ipHE_LIKE][ipHELIUM] = 6;
		/* And n=5 for carbon, nitrogen, oxygen, and iron because they are quite abundant */
		iso.n_HighestResolved_max[ipHE_LIKE][ipCARBON] = 5;
		iso.n_HighestResolved_max[ipHE_LIKE][ipNITROGEN] = 5;
		iso.n_HighestResolved_max[ipHE_LIKE][ipOXYGEN] = 5;
		iso.n_HighestResolved_max[ipHE_LIKE][ipIRON] = 5;
		iso.n_HighestResolved_max[ipHE_LIKE][ipZINC] = 5;

		for( nelem=ipHELIUM; nelem < LIMELM; ++nelem )
		{
			/* at the very least do n=3 for every element that is turned on.	*/
			iso.n_HighestResolved_max[ipHE_LIKE][nelem] = MAX2( iso.n_HighestResolved_max[ipHE_LIKE][nelem], 3 );
			/* Calculate total levels for each element.	*/
			iso.numLevels_max[ipHE_LIKE][nelem] = ( ( 1 + iso.n_HighestResolved_max[ipHE_LIKE][nelem] ) * iso.n_HighestResolved_max[ipHE_LIKE][nelem] + 1 );
			/* number of levels to print */
			iso.numPrintLevels[ipHE_LIKE][nelem] = iso.numLevels_max[ipHE_LIKE][nelem]-1;
		}
	}

	/* number of electrons in valence shell that can compton recoil ionize */
	{
		long int nCom[LIMELM] = 
		{
			1 , 2 ,								/* K 1s shell */
			1 , 2 ,								/* L 2s shell */
			1 , 2 , 3 , 4 , 5 , 6 ,				/* L 2p shell */
			1 , 2 ,								/* M 3s shell */
			1 , 2 , 3 , 4 , 5 , 6 ,				/* M 3p shell */
			1 , 2 , 3 , 4 , 5 , 6 , 7 , 8 ,		/* M 3d shell */
			1 , 2 ,								/* N 4s shell */
			1 , 2								/* N 4p shell */
		};

		for( nelem=ipHYDROGEN; nelem<LIMELM; ++nelem )
		{
			ionbal.nCompRecoilElec[nelem] = nCom[nelem];
		}
	}
}
