/* This file is part of Cloudy and is copyright (C) 1978-2007 by Gary J. Ferland.
 * For conditions of distribution and use, see copyright notice in license.txt */
/*gridXspec handles all grid calculations, called by griddo */
/*gridFunc */
/*GridGather - gathers spectra for each grid calculation to save for end */
/*GridInitialize - init grid calculations */
#include "cddefines.h"
#include "punch.h"
#include "optimize.h"
#include "cddrive.h"
#include "rfield.h"
#include "grid.h"
#include "called.h"
#include "prt.h"

static int FirstRun = true;

/*static void gridFunc(float param[] );*/

/*gridXspec handles all grid calculations, called by griddo */
void gridXspec(float xc[], long int nInterpVars)
{
	long int i, j, k;
	double averageChi2;
	
	DEBUG_ENTRY( "grid_xspec()" );
	
	if( nInterpVars > LIMPAR )
	{
		fprintf( ioQQQ, "grid_do: too many parameters are varied, increase LIMPAR\n" );
		puts( "[Stop]" );
		cdEXIT(EXIT_FAILURE);
	}

	optimize.nOptimiz = 0;
	
	grid.nintparm = nInterpVars;
	
	/* if this is changed there must be some change made to actually
	 * stuff the additional parameter information. */
	grid.naddparm = 0;

	ASSERT( grid.nintparm + grid.naddparm <= LIMPAR );

	grid.totNumModels = 1;
	/* >>chng 06 aug 21, allow the number of parameter values to be different for different parameters. */
	for( i=0; i<nInterpVars; i++ )
	{
		/* >>chng 06 sep 4, use grid variable instead of passing to routine. */
		grid.totNumModels *= grid.numParamValues[i];
	}
	/* grid.totNumModels = (long)pow((double)nParVals, (double)nInterpVars); */
	ASSERT( grid.totNumModels > 1 );

	grid.paramNames = (char**)MALLOC(sizeof(char*)*(unsigned)(nInterpVars+grid.naddparm) );

	grid.paramMethods = (long*)MALLOC(sizeof(long)*(unsigned)(nInterpVars+grid.naddparm) );

	grid.paramRange = (float**)MALLOC(sizeof(float*)*(unsigned)(nInterpVars+grid.naddparm) );

	grid.paramData = (float**)MALLOC(sizeof(float*)*(unsigned)(nInterpVars+grid.naddparm) );

	grid.interpParameters = (float**)MALLOC(sizeof(float*)*(unsigned)(grid.totNumModels) );

	for( i=0; i<nInterpVars+grid.naddparm; i++ )
	{
		grid.paramNames[i] = (char*)MALLOC(sizeof(char)*(unsigned)(12) );

		grid.paramRange[i] = (float*)MALLOC(sizeof(float)*(unsigned)(6) );

		grid.paramData[i] = (float*)MALLOC(sizeof(float)*(unsigned)(grid.numParamValues[i]) );

		sprintf( grid.paramNames[i],	"%s%ld", "PARAM", i+1 );
		/* Method is 0 for linear, 1 for logarithmic, for now all linear. */
		grid.paramMethods[i] = 0;
		/* Initial */
		grid.paramRange[i][0] = xc[i]+grid.paramIncrements[i]*(grid.numParamValues[i]-1.f)/2.f;
		/* Delta */
		grid.paramRange[i][1] = grid.paramIncrements[i]/10.f;
		/* Minimum */
		grid.paramRange[i][2] = xc[i]-grid.paramIncrements[i]/10.f;
		/* Bottom */
		grid.paramRange[i][3] = xc[i]-grid.paramIncrements[i]/10.f;
		/* Top */
		grid.paramRange[i][4] = xc[i]+grid.paramIncrements[i]*(grid.numParamValues[i]-1.f)+grid.paramIncrements[i]/10.f;
		/* Maximum */
		grid.paramRange[i][5] = xc[i]+grid.paramIncrements[i]*(grid.numParamValues[i]-1.f)+grid.paramIncrements[i]/10.f;
		
		for( j=0; j<grid.numParamValues[i]; j++ )
		{
			grid.paramData[i][j] = xc[i]+grid.paramIncrements[i]*j;
		}
	}

	for( i=0; i<grid.totNumModels; i++ )
	{
		grid.interpParameters[i] = (float*)MALLOC(sizeof(float)*(unsigned)(nInterpVars) );
	}

	/* >>chng 06 aug 23, this logic now allows non-square parameter spaces.  */
	for( i=0; i< grid.totNumModels; i++ )
	{
		float variableVector[LIMPAR];

		for( j=0; j<nInterpVars; j++ )
		{
			int index;
			long volumeOtherDimensions = 1;
			
			/* by "volume", we simply mean the product of the parameter dimensions 
			 * AFTER the present index, i.e.:
			 * first "volume" is product of grid.numParamValues[1]*grid.numParamValues[2]*....grid.numParamValues[n]
			 * second "volume" is product of grid.numParamValues[2]*grid.numParamValues[3]*....grid.numParamValues[n]
			 * last "volume" is unity.  */
			for( k=j+1; k<nInterpVars; k++ )
			{
				volumeOtherDimensions *= grid.numParamValues[k];
			}

			/* For each successive parameter, the "volume" is less than the previous one.
			 * So the left-hand side of this modulus operation increases for each parameter,
			 * which causes the index of each parameter to be incremented more often than the
			 * index of the previous parameter.  Thus, the last dimension is incremented
			 * every time, then the second to last dimension is incremented with each repeat
			 * of the last dimension.  This repeats until, finally, the first dimension is
			 * incremented.  For example, the indices of the parameter vectors for a 2x2x3
			 * box would be ordered as such:
			 * [0][0][0]
			 * [0][0][1]
			 * [0][0][2]
			 * [0][1][0]
			 * [0][1][1]
			 * [0][1][2]
			 * [1][0][0]
			 * [1][0][1]
			 * [1][0][2]
			 * [1][1][0]
			 * [1][1][1]
			 * [1][1][2]
			 */
			index = (int)( (i/volumeOtherDimensions)%(grid.numParamValues[j]) );

			variableVector[j] = xc[j] + grid.paramIncrements[j]*index;

			grid.interpParameters[i][j] = variableVector[j];
		}
		for( j=nInterpVars; j<LIMPAR; j++ )
		{
			variableVector[j] = xc[j];
		}

		if( i == grid.totNumModels - 1 )
		{
			called.lgTalk = true;
			called.lgTalkIsOK = true;
			prt.lgFaintOn = true;
			punch.lgOpenUnits = true;
			grid.lgGridDone = true;
		}

		averageChi2 = optimize_func(variableVector);
		/* silly test to use the var averageChi2 for something - keeps lint and some
		 * compilers happy */
		if( averageChi2 < 0. && averageChi2 == 0 )
			fprintf( ioQQQ , "DEBUG interesting impossible print has occurred.\n");
		/* gridFunc(variableVector); */

		ASSERT( FirstRun == false );

	}

	DEBUG_EXIT( "grid_xspec()" );
	return;
}

/*gridFunc 
\todo	0	does this do anything? */
#if 0
static void gridFunc(float param[] )
{
	bool lgBAD, lgLimOK;

	long int i;

	double snorm;

	DEBUG_ENTRY( "grid_func()" );
	/* limpar defined in optimize.h */

	/* This routine is called by AMOEBA (or POWELL) with values of the
	 * variable parameters for CLOUDY in the array p(i). It returns
	 * the value FUNC = SUM (obs-model)**2/sig**2 for the lines
	 * specified in the observational data file, values held in the
	 * common blocks /OBSLIN/ & /OBSINT/
	 * replacement input strings for CLOUDY READR held in /chCardSav/
	 * parameter information for setting chCardSav held in /parmv/
	 * additional variables
	 * Gary's variables
	 */

	/* initialize the code for this run */
	/* cdInit();*/
	/* zero out lots of variables */
	zero();

	if( optimize.lgOptimFlow )
	{
		fprintf( ioQQQ, " trace, grid_func variables" );
		for( i=0; i < optimize.nvary; i++ )
		{
			fprintf( ioQQQ, "%10.2e", param[i] );
		}
		fprintf( ioQQQ, "\n" );
	}

	for( i=0; i < optimize.nvary; i++ )
	{
		optimize.vparm[0][i] = param[i];
	}

	/* always increment nOptimiz, even if parameters are out of bounds,
	 * this prevents grid to get stuck in infinite loop */
	++optimize.nOptimiz;

	/* call routine to pack /kardsv/ variable with appropriate
	 * CLOUDY input lines given the array of variable parameters p(i) */
	vary_input(&lgLimOK);

	if( !lgLimOK )
	{
		/* these parameters are not within limits of parameter search
		 * >>chng 96 apr 26, as per Peter van Hoof comment */
		fprintf( ioQQQ, " Iteration %ld not within range.\n", 
		  optimize.nOptimiz );

		cdEXIT(EXIT_FAILURE);
	}

	for( i=0; i < optimize.nvary; i++ )
	{
		optimize.varmax[i] = (float)MAX2(optimize.varmax[i],optimize.vpused[i]);
		optimize.varmin[i] = (float)MIN2(optimize.varmin[i],optimize.vpused[i]);
	}

	lgBAD = cloudy();

	if( lgBAD )
	{
		fprintf( ioQQQ, " Cloudy returned error condition - what happened?\n" );
	}

	GridInitialize();

	if( LineSave.ipNormWavL < 0 )
	{
		fprintf( ioQQQ, 
			" Normalization line array index is bad.  What has gone wrong?\n" );
		puts( "[Stop in grid_func]" );
		cdEXIT(EXIT_FAILURE);
	}
	snorm = LineSv[LineSave.ipNormWavL].sumlin[LineSave.lgLineEmergent];

	if( snorm == 0. )
	{
		fprintf( ioQQQ, "\n\n PROBLEM Normalization line has zero intensity.  What has gone wrong?\n" );
		fprintf( ioQQQ, " Is spectrum normalized to a species that does not exist?\n" );
		puts( "[Stop in grid_func]" );
		cdEXIT(EXIT_FAILURE);
	}

	/* first print all warnings */
	cdWarnings(ioQQQ);

	DEBUG_EXIT( "grid_func()" );
	return;
}
#endif

/*GridInitialize - init grid calcs */
void GridInitialize( void )
{
	long i,j;

	DEBUG_ENTRY( "GridInitialize()" );

	/* Do this stuff once.  malloc some arrays, and get the energies. */
	if( FirstRun )
	{
		grid.numEnergies = rfield.nupper-2;
			
		grid.Energies = (float*)MALLOC(sizeof(float)*(unsigned)(grid.numEnergies) );

		grid.Spectra = (float***)MALLOC(sizeof(float**)*(unsigned)(NUM_OUTPUT_TYPES) );

		for( i=0; i< NUM_OUTPUT_TYPES; i++ )
		{
			/* if( grid.lgOutputTypeOn[i] ) */
			{
				grid.Spectra[i] = (float**)MALLOC(sizeof(float*)*(unsigned)(grid.totNumModels) );

				for( j=0; j<grid.totNumModels; j++ )
				{
					grid.Spectra[i][j] = (float*)MALLOC(sizeof(float)*(unsigned)(grid.numEnergies) );
				}
			}
		}

		for( i=0; i<grid.numEnergies; i++ )
		{
			grid.Energies[i] = rfield.AnuOrg[i];
			grid.Energies[i] = rfield.AnuOrg[i];
		}

		FirstRun = false;
	}

	for( i=1; i< NUM_OUTPUT_TYPES; i++ )
	{
		/* if( grid.lgOutputTypeOn[i] ) */
		{
			/* Grab spectrum for xspec printout */
			cdSPEC2( i, grid.numEnergies, grid.Spectra[i][optimize.nOptimiz-1]);
		}
	}

	DEBUG_EXIT( "grid_func()" );

	return;
}

/*GridGather - gathers spectra for each grid calc to save for end */
void GridGather( void )
{
	DEBUG_ENTRY( "GridGather()" );

	long i;

	if( optimize.nOptimiz==grid.totNumModels && grid.lgGrid )
	{
		ASSERT( optimize.nOptimiz >= 1 );
		for( i=1; i< NUM_OUTPUT_TYPES; i++ )
		{
			/* if( grid.lgOutputTypeOn[i] ) */
			{
				/* Grab spectrum for xspec printout */
				cdSPEC2( i, grid.numEnergies, grid.Spectra[i][optimize.nOptimiz-1]);
			}
		}
	}

	DEBUG_EXIT( "GridGather()" );

	return;
}
