/* This file is part of Cloudy and is copyright (C)1978-2007 by Gary J. Ferland
 * For conditions of distribution and use see copyright notice in license.txt */
/*prt_LineLabels punch all labels and wavelengths for emission line array */
/*sprt_wl write wavelength to string - must be kept parallel with prt_wl */
/*prt_wl - print floating wavelength in Angstroms, in output format */
#include "cddefines.h"
#include "lines.h"
#include "prt.h"

/*prt_wl print floating wavelength in Angstroms, in output format */
void prt_wl( FILE *ioOUT , float wl )
{
	char chUnits;

	DEBUG_ENTRY( "prt_wl()" );

	/* print in A unless > 1e4, then use microns */
	if( wl > 1e8 )
	{
		/* centimeters */
		chUnits = 'c';
		wl /= 1e8;
	}
	else if( wl > 1e4 )
	{
		/* microns */
		chUnits = 'm';
		wl /= 1e4;
	}
	else if( wl == 0. )
	{
		chUnits = ' ';
	}
	else
	{
		/* Angstroms units */
		chUnits = 'A';
	}

	/* want total of four sig figs */
	if( LineSave.sig_figs == 4 )
	{
		if( wl==0. )
		{
			fprintf(ioOUT, "%5i", 0 );
		}
		else if( wl<10. )
		{
			fprintf(ioOUT, "%5.3f", wl );
		}
		else if( wl<100. )
		{
			fprintf(ioOUT, "%5.2f", wl );
		}
		else if( wl < 1e3 )
		{
			fprintf(ioOUT, "%5.1f", wl );
		}
		else if( wl < 1e4 )
		{
			fprintf(ioOUT, "%5.0f", wl );
		}
		else if( wl < 1e5 )
		{
			fprintf(ioOUT, "%5i", (int)wl );
		}
		else
		{
			TotalInsanity();
		}
	}
	else if( LineSave.sig_figs == 5 )
	{
		/* this branch five sig figs */
		if( wl==0. )
		{
			fprintf(ioOUT, "%5i", 0 );
		}
		else if( wl<10. )
		{
			fprintf(ioOUT, "%5.4f", wl );
		}
		else if( wl<100. )
		{
			fprintf(ioOUT, "%5.3f", wl );
		}
		else if( wl < 1e3 )
		{
			fprintf(ioOUT, "%5.2f", wl );
		}
		else if( wl < 1e4 )
		{
			fprintf(ioOUT, "%5.1f", wl );
		}
		else if( wl < 1e5 )
		{
			fprintf(ioOUT, "%5.0f", wl );
		}
		else if( wl < 1e6 )
		{
			fprintf(ioOUT, "%5i", (int)wl );
		}
		else
		{
			TotalInsanity();
		}
	}
	else
	{
		ASSERT( LineSave.sig_figs == 6 );
		/* this branch five sig figs */
		if( wl==0. )
		{
			fprintf(ioOUT, "%6i", 0 );
		}
		else if( wl<10. )
		{
			fprintf(ioOUT, "%6.5f", wl );
		}
		else if( wl<100. )
		{
			fprintf(ioOUT, "%6.4f", wl );
		}
		else if( wl < 1e3 )
		{
			fprintf(ioOUT, "%6.3f", wl );
		}
		else if( wl < 1e4 )
		{
			fprintf(ioOUT, "%6.2f", wl );
		}
		else if( wl < 1e5 )
		{
			fprintf(ioOUT, "%6.1f", wl );
		}
		else if( wl < 1e6 )
		{
			fprintf(ioOUT, "%6.0f", wl );
		}
		else if( wl < 1e7 )
		{
			fprintf(ioOUT, "%6i", (int)wl );
		}
		else
		{
			TotalInsanity();
		}
	}
	fprintf(ioOUT, "%c", chUnits );

	DEBUG_EXIT( "prt_wl()" );

	return;
}

/* write wavelength to string - must be kept parallel with prt_wl */
void sprt_wl( char *chString , float wl )
{
	char chUnits;

	DEBUG_ENTRY( "sprt_wl()" );

	/* print in A unless > 1e4, then use microns */
	if( wl > 1e8 )
	{
		/* centimeters */
		chUnits = 'c';
		wl /= 1e8;
	}
	else if( wl > 1e4 )
	{
		/* microns */
		chUnits = 'm';
		wl /= 1e4;
	}
	else if( wl == 0. )
	{
		chUnits = ' ';
	}
	else
	{
		/* Angstroms units */
		chUnits = 'A';
	}

	/* want total of four sig figs */
	if( LineSave.sig_figs == 4 )
	{
		if( wl==0. )
		{
			sprintf(chString, "%5i", 0 );
		}
		else if( wl<10. )
		{
			sprintf(chString, "%5.3f", wl );
		}
		else if( wl<100. )
		{
			sprintf(chString, "%5.2f", wl );
		}
		else if( wl < 1e3 )
		{
			sprintf(chString, "%5.1f", wl );
		}
		else if( wl < 1e4 )
		{
			sprintf(chString, "%5.0f", wl );
		}
		else if( wl < 1e5 )
		{
			sprintf(chString, "%5i", (int)wl );
		}
		else
		{
			TotalInsanity();
		}
		chString[6] = chUnits;
	}
	else if( LineSave.sig_figs == 5 )
	{
		/* this branch five sig figs */
		if( wl==0. )
		{
			sprintf(chString, "%5i", 0 );
		}
		else if( wl<10. )
		{
			sprintf(chString, "%5.4f", wl );
		}
		else if( wl<100. )
		{
			sprintf(chString, "%5.3f", wl );
		}
		else if( wl < 1e3 )
		{
			sprintf(chString, "%5.2f", wl );
		}
		else if( wl < 1e4 )
		{
			sprintf(chString, "%5.1f", wl );
		}
		else if( wl < 1e5 )
		{
			sprintf(chString, "%5.0f", wl );
		}
		else if( wl < 1e6 )
		{
			sprintf(chString, "%5i", (int)wl );
		}
		else
		{
			TotalInsanity();
		}
		chString[6] = chUnits;
	}
	else
	{
		ASSERT( LineSave.sig_figs == 6 );
		/* this branch five sig figs */
		if( wl==0. )
		{
			sprintf(chString, "%6i", 0 );
		}
		else if( wl<10. )
		{
			sprintf(chString, "%6.5f", wl );
		}
		else if( wl<100. )
		{
			sprintf(chString, "%6.4f", wl );
		}
		else if( wl < 1e3 )
		{
			sprintf(chString, "%6.3f", wl );
		}
		else if( wl < 1e4 )
		{
			sprintf(chString, "%6.2f", wl );
		}
		else if( wl < 1e5 )
		{
			sprintf(chString, "%6.1f", wl );
		}
		else if( wl < 1e6 )
		{
			sprintf(chString, "%6.0f", wl );
		}
		else if( wl < 1e7 )
		{
			sprintf(chString, "%6i", (int)wl );
		}
		else
		{
			TotalInsanity();
		}
		chString[7] = chUnits;
	}
	

	DEBUG_EXIT( "sprt_wl()" );

	return;
}

/*prt_LineLabels punch all labels and wavelengths for emission line array */
void prt_LineLabels(FILE * ioOUT)
{
	long int i;

	DEBUG_ENTRY( "prt_LineLabels()" );

	for( i=0; i < LineSave.nsum; i++ )
	{
		/* >>chng 02 jun 10, punch everything */
#		if 0
		/* do not print all these redundant labels */
		if( !strcmp( LineSv[i].chALab , "Inwd" ) ||
			!strcmp( LineSv[i].chALab , "Coll" ) ||
			!strcmp( LineSv[i].chALab , "Pump" ) ||
			!strcmp( LineSv[i].chALab , "Heat" ) )
			continue;
#		endif
		if( strcmp( LineSv[i].chALab , "####" )==0 )
		{
			/*fprintf( ioOUT, "%s ", LineSv[i].chALab );*/
			fprintf( ioOUT, "####\t%s",LineSave.chHoldComments[(int)LineSv[i].wavelength] ); 
		}
		else
		{
			/* this format chosen to be identical to that used by final */
			fprintf( ioOUT, "%li\t%s\t", 
				i,
				LineSv[i].chALab );
			prt_wl( ioOUT, LineSv[i].wavelength );
		}
		fprintf( ioOUT, "\n" );
	}

	DEBUG_EXIT( "sprt_wl()" );
	return;
}

